package org.lejos.ev3.recognitionclient;

import java.io.File;
import java.io.IOException;

import lejos.hardware.Brick;
import lejos.hardware.BrickFinder;
import lejos.hardware.Button;
import lejos.hardware.device.DeviceIdentifier;
import lejos.hardware.lcd.TextLCD;
import lejos.hardware.motor.NXTRegulatedMotor;
import lejos.hardware.port.SensorPort;
import lejos.hardware.sensor.EV3ColorSensor;
import lejos.robotics.Color;
import lejos.utility.Delay;

import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.highgui.Highgui;
import org.opencv.highgui.VideoCapture;


/**
 * @author 		Jarema Czajkowski
 * @created 	10-06-2016
 * @license 	GNU General Public License
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */ 
public class RecognitionClient {
    private static final int frame_width = 320;
    private static final int frame_height= 240;
    private static final int msg_x = 1;
    private static final int msg_y = 1;
    private static final int minimalSpace = 1024*1000*10;
    private static final String fname_prefix = "camera_result_";
    
	public static void main(String[] args) throws IOException {
		System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
				
        Brick EV3Brick = BrickFinder.getDefault();
        TextLCD lcd = EV3Brick.getTextLCD();
        
		lcd.drawString("Put color sensor", msg_x, 1);  
		lcd.drawString("in port 4 and", msg_x, 2);  
		lcd.drawString("servo in port A,B", msg_x, 3);  
		lcd.drawString("then press key", msg_x, 4);  
		if (Button.waitForAnyPress() == Button.ID_ESCAPE)
	        System.exit(1);  	        
		
		lcd.clear();
		lcd.drawString("Initializing...", msg_x, msg_y);            
		Mat frame = new Mat();
        VideoCapture vid = new VideoCapture(0);
        
		NXTRegulatedMotor myLMotor = new NXTRegulatedMotor(EV3Brick.getPort("B"));
		NXTRegulatedMotor mySMotor = new NXTRegulatedMotor(EV3Brick.getPort("A"));
		
		DeviceIdentifier devSensor = new DeviceIdentifier(SensorPort.S4);
		boolean exists = devSensor.getPortType() == 122;
		devSensor.close();
		
		if (exists) {
			EV3ColorSensor mySensor = new EV3ColorSensor(SensorPort.S4);
			
			vid.set(Highgui.CV_CAP_PROP_FRAME_WIDTH, frame_width);
			vid.set(Highgui.CV_CAP_PROP_FRAME_HEIGHT, frame_height);
			vid.open(0);
	
			lcd.clear(msg_y);
			lcd.drawString("Waiting...", msg_x, msg_y); 
			
			int ident = Color.NONE; int fileCount = 1;
			do {			
				ident = mySensor.getColorID(); 
				Delay.msDelay(1000);
			} while(ident == Color.NONE || ident == Color.BLACK);
			 
			do {
				lcd.drawString("Transporting...", msg_x, msg_y);        
				Button.LEDPattern(3);
				
				myLMotor.rotate( -360*3 ,true );
				mySMotor.rotate( 360*5 );
				
				myLMotor.setSpeed(150);
				mySMotor.setSpeed(150);
				
				lcd.clear(msg_y);
		        lcd.drawString("Processing...", msg_x, msg_y); 
				Button.LEDPattern(2);
				String fname = fname_prefix+fileCount+".jpg";
				if (vid.isOpened()) {
					vid.read(frame);
					if (!frame.empty()) {
						Highgui.imwrite(fname,frame);
						
						File myFile = new File("/home/lejos/programs/"+fname);
						fileCount += 1;
						if ( myFile.exists() ) {
							long diskUsage= myFile.getUsableSpace();
							if (diskUsage < minimalSpace)  System.exit(1);
						}
					}	
				}
		        Button.LEDPattern(1);
		        lcd.drawString("Ready for Next...", msg_x, msg_y);     
			} while (Button.waitForAnyPress() != Button.ID_ESCAPE);
			
	        myLMotor.close();
	        mySMotor.close();
	        mySensor.close();
		}
    }

}
