package org.lejos.ev3.recognitionserver;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.*;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import lejos.remote.ev3.RMIMenu;


/**
 * @author 		Jarema Czajkowski
 * @created 	07-06-2016
 * @license 	GNU General Public License
 *
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */ 
public class RecognitionServer {
	private static final String PROGRAMS_DIR = "/home/lejos/programs/";
	private static final String fname_prefix = "camera_result_";
	
	private static String brickIP;
	private static RMIMenu menu;
	
    public static void main(String[] args)  {
    	brickIP = "10.0.1.1"; 
    	menu = null;
    	
    	System.out.println("Try connect to " + brickIP);
    	try {
			menu = (RMIMenu) Naming.lookup("//" + brickIP + "/RemoteMenu");
		} catch (IOException | NotBoundException ex) {
			System.out.println("No EV3 Found.");
		}
    	if (menu != null) {
    		try {
    			System.out.println("Storage location is " +  Paths.get(System.getenv("USERPROFILE"), menu.getName()).toString());
				System.out.println("Connected to " + menu.getName());
					
	    		menu.runProgram("RecognitionClient");
	    		System.out.println("Plase wait. Processing...");
	    		
	    		String data = "START";
				while (data != null) {
					data = menu.getExecutingProgramName();
	    		}
	    		
				Path dirpath = Paths.get(System.getenv("USERPROFILE"), menu.getName());
				if(!Files.exists(dirpath)) {
					Files.createDirectory(dirpath);
				}
				
				int fileCount = 1;
	    		String fname; 
				Path filepath = null;
				do {
	    			fname = fname_prefix+fileCount+".jpg";
					filepath = Paths.get(System.getenv("USERPROFILE"), menu.getName(), fname);
	    			fileCount += 1;
	    		} while (getFile(filepath.toFile(), fname));
    		} catch (IOException ex) {
    			System.out.println("Comunication error.");
    		}    	
    	}
    	System.out.println("Thank you. Finished.");
    }
    
	/**
	 * Download a file from the EV3
	 * @return 
	 */
	private static boolean getFile(File file, String fileName) {
		FileOutputStream out = null;
		try {
			long fileSize = menu.getFileSize(PROGRAMS_DIR + "/" + fileName);
			if (fileSize < 1) return false;
			
			System.out.println("Fetching file " + fileName);
			out = new FileOutputStream(file);
			byte[] data = menu.fetchFile(PROGRAMS_DIR + "/" + fileName);
			out.write(data);
			out.close();		
			return true;
		} catch (IOException ex) {
			return false;
		}    
	}
}
